/* Copyright 2016 Juan Pablo Lozano
*
* This file is part of GCleaner.
*
* GCleaner is free software: you can redistribute it
* and/or modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation, either version 3 of the
* License, or (at your option) any later version.
*
* GCleaner is distributed in the hope that it will be
* useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
* Public License for more details.
*
* You should have received a copy of the GNU General Public License along
* with GCleaner. If not, see http://www.gnu.org/licenses/.
*/

//Importing libraries GTK+, GLib and Granite
using Gtk;
using GLib;
using Granite;

namespace GCleaner.Widgets
{
	public class Header : HeaderBar
	{

		public GCleaner.App app;

		public Header (GCleaner.App app)
		{
			this.app = app;

			//Variables
			string complete_system_specs;

			//GCleaner icon
			Image icon = new Image ();
			try
			{
				var icon_pixbuf = new Gdk.Pixbuf.from_file_at_scale ("/usr/share/pixmaps/gcleanerhb.svg", 32, 32, false);
				icon.set_from_pixbuf (icon_pixbuf);
			  } catch (GLib.Error e) {
				stderr.printf ("COM.GCLEANER.APP.HEADERBAR: [GLIB::ERROR CREANDO ICONO PIXBUF]\n");
				stderr.printf (">>> Comprobar ruta: /usr/share/pixmaps/gcleanerhb.svg\n");
			}

			complete_system_specs = getProcessor () + "  •  " + getMemory () + " RAM  •  " + getGraphics ();

			//LABELS WITH PANGO MARKUP
			Label title = new Label ("");
			title.set_markup ("<b>GCleaner</b>");
			
			Label version = new Label ("");
			version.set_markup ("<small>v" + Constants.VERSION + "</small>");

			Label os_information = new Label ("");
			os_information.set_markup ("<span>" + getOS () + "</span>");//font_size='small' by default

			Label system_specs = new Label ("");
			system_specs.set_markup ("<span font_size='small'>" + complete_system_specs + "</span>");


			//BUTTONS
			var menu = new Gtk.Menu ();
			var appmenu = this.app.create_appmenu (menu);

			//BOXES
			/* to assemble the header */
			Box container_box 	= new Box (Orientation.HORIZONTAL, 4);
			Box nameVersion_box = new Box (Orientation.VERTICAL, 0);
			Box specsOS_box 	= new Box (Orientation.VERTICAL, 0);

			//Cajas por elementos
			Box icon_box 	= new Box (Orientation.HORIZONTAL, 0);
			Box app_box 	= new Box (Orientation.HORIZONTAL, 0);
			Box version_box = new Box (Orientation.HORIZONTAL, 0);
			Box os_box 		= new Box (Orientation.HORIZONTAL, 0);
			Box specs_box 	= new Box (Orientation.HORIZONTAL, 0);

			//PACKAGING
			/* Packaging widgets */
			icon_box.pack_start (icon, false, true, 0);
			app_box.pack_start (title, false, true, 0);
			version_box.pack_start (version, false, true, 0);
			os_box.pack_start (os_information, false, true, 0);
			specs_box.pack_start (system_specs, false, true, 0);

			/* Packaging boxes */
			nameVersion_box.pack_start (app_box, false, true, 0);
			nameVersion_box.pack_start (version_box, false, true, 0);
			specsOS_box.pack_start (os_box, false, true, 0);
			specsOS_box.pack_start (specs_box, false, true, 0);
			container_box.pack_start (icon_box, false, true, 6);
			container_box.pack_start (nameVersion_box, false, true, 6);
			container_box.pack_start (specsOS_box, false, true, 6);


			ToolItem item = new ToolItem ();
			item.add (container_box);


			/* HeaderBar properties */
			this.pack_start (item);
			this.pack_end (appmenu);
			this.set_show_close_button (true);

			this.show_all ();
		}	
	}
}